@@ -23,13 +23,22 @@
 # or write to the Free Software Foundation, Inc., 
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-
+from __future__ import print_function, division, absolute_import
 import copy
 import math
 
 import numpy
 import os.path
 
+try:
+    from PyQt5.Qt import QObject
+    from PyQt5.Qt import pyqtSignal
+except ImportError:
+    qt_available = False
+    pass
+else:
+    qt_available = True
+
 from Tigger import startup_dprint
 
 startup_dprint(1, "starting ModelClasses")
@@ -40,6 +49,22 @@ AtomicTypes = dict(bool=bool, int=int, float=float, complex=complex, str=str, li
                    NoneType=lambda x: None)
 
 
+if qt_available:
+    class ModelItemSignals(QObject):
+        """ModelItemSignals is a connecting object for adding pyqtSignals to ModelItem.
+        The signals are compatible with PyQt 5 and Tigger v1.6.0
+        """
+
+        updated = pyqtSignal(int, object)
+        changeCurrentSource = pyqtSignal(object, object, object)
+        selected = pyqtSignal(object, object)
+        changeGroupingStyle = pyqtSignal(object, object)
+        changeGroupingVisibility = pyqtSignal(object, object)
+
+        def __init__(self):
+            QObject.__init__(self)
+
+
 class ModelItem(object):
     """ModelItem is a base class for all model items. ModelItem provides functions
     for saving, loading, and initializing items, using class attributes that describe the
@@ -80,34 +105,34 @@ class ModelItem(object):
         for attr, value in zip(self.mandatory_attrs, args):
             if not isinstance(value, AllowedTypesTuple):
                 raise TypeError("invalid type %s for attribute %s (class %s)" % (
-                type(value).__name__, attr, self.__class__.__name__))
+                    type(value).__name__, attr, self.__class__.__name__))
             setattr(self, attr, value)
         # set optional attributes from keywords
-        for kw, default in self.optional_attrs.items():
+        for kw, default in list(self.optional_attrs.items()):
             value = kws.pop(kw, default)
             if not isinstance(value, AllowedTypesTuple):
                 raise TypeError("invalid type %s for attribute %s (class %s)" % (
-                type(value).__name__, kw, self.__class__.__name__))
+                    type(value).__name__, kw, self.__class__.__name__))
             setattr(self, kw, value)
         # set extra attributes, if any are left
         self._extra_attrs = set()
         if self.allow_extra_attrs:
-            for kw, value in kws.items():
+            for kw, value in list(kws.items()):
                 if not isinstance(value, AllowedTypesTuple):
                     raise TypeError("invalid type %s for attribute %s (class %s)" % (
-                    type(value).__name__, kw, self.__class__.__name__))
+                        type(value).__name__, kw, self.__class__.__name__))
                 self.setAttribute(kw, value)
         elif kws:
             raise TypeError("unknown parameters %s in constructor of %s" % (
-            ','.join(list(kws.keys())), self.__class__.__name__))
+                ','.join(list(kws.keys())), self.__class__.__name__))
         # other init
         self._signaller = None
         self._connections = set()
 
     def enableSignals(self):
         """Enables Qt signals for this object."""
-        import PyQt4.Qt
-        self._signaller = PyQt4.Qt.QObject()
+        if qt_available:
+            self._signaller = ModelItemSignals()
 
     def signalsEnabled(self):
         return bool(self._signaller)
@@ -116,17 +141,33 @@ class ModelItem(object):
         """Connects SIGNAL from object to specified receiver slot. If reconnect is True, allows duplicate connections."""
         if not self._signaller:
             raise RuntimeError("ModelItem.connect() called before enableSignals()")
-        import PyQt4.Qt
         if reconnect or (signal_name, receiver) not in self._connections:
             self._connections.add((signal_name, receiver))
-            PyQt4.Qt.QObject.connect(self._signaller, PyQt4.Qt.SIGNAL(signal_name), receiver)
+            if signal_name == 'updated':
+                self._signaller.updated.connect(receiver)
+            elif signal_name == 'changeCurrentSource':
+                self._signaller.changeCurrentSource.connect(receiver)
+            elif signal_name == 'selected':
+                self._signaller.selected.connect(receiver)
+            elif signal_name == 'changeGroupingStyle':
+                self._signaller.changeGroupingStyle.connect(receiver)
+            elif signal_name == 'changeGroupingVisibility':
+                self._signaller.changeGroupingVisibility.connect(receiver)
 
     def emit(self, signal_name, *args):
         """Emits named SIGNAL from this object ."""
         if not self._signaller:
             raise RuntimeError("ModelItem.emit() called before enableSignals()")
-        import PyQt4.Qt
-        self._signaller.emit(PyQt4.Qt.SIGNAL(signal_name), *args)
+        if signal_name == 'updated':
+            self._signaller.updated.emit(*args)
+        elif signal_name == 'changeCurrentSource':
+            self._signaller.changeCurrentSource.emit(*args)
+        elif signal_name == 'selected':
+            self._signaller.selected.emit(*args)
+        elif signal_name == 'changeGroupingStyle':
+            self._signaller.changeGroupingStyle.emit(*args)
+        elif signal_name == 'changeGroupingVisibility':
+            self._signaller.changeGroupingVisibility.emit(*args)
 
     def registerClass(classobj):
         if not isinstance(classobj, type):
@@ -154,7 +195,7 @@ class ModelItem(object):
     def getAttributes(self):
         """Returns list of all attributes (mandatory+optional+extra), as (attr,value) tuples"""
         attrs = [(attr, getattr(self, attr)) for attr in self.mandatory_attrs]
-        for attr, default in self.optional_attrs.items():
+        for attr, default in list(self.optional_attrs.items()):
             val = getattr(self, attr, default)
             if val != default:
                 attrs.append((attr, val))
@@ -210,11 +251,11 @@ class ModelItem(object):
         and rem_tags is to be passed to child items' resolveMarkup() """
         # figure out enclosing tag
         if not tags:
-            tag, tags = None, None;  # use default
+            tag, tags = None, None  # use default
         elif isinstance(tags, str):
-            tag, tags = tags, None;  # one tag supplied, use that here and use defaults for sub-items
+            tag, tags = tags, None  # one tag supplied, use that here and use defaults for sub-items
         elif isinstance(tags, (list, tuple)):
-            tag, tags = tags[0], tags[1:];  # stack of tags supplied: use first here, pass rest to sub-items
+            tag, tags = tags[0], tags[1:]  # stack of tags supplied: use first here, pass rest to sub-items
         else:
             raise ValueError("invalid 'tags' parameter of type " + str(type(tags)))
         # if tag is None, use default
@@ -287,7 +328,7 @@ class ModelItem(object):
             markup += "mdlattr=\"%s\" " % attr
         # if rendering table row, use TD to render comments
         if verbose is None:
-            verbose = attr;  # and self.attr_verbose.get(attr)
+            verbose = attr  # and self.attr_verbose.get(attr)
         if tag == "TR":
             comment = "<TD bgcolor=yellow>%s</TD>"
         else:
@@ -433,14 +474,14 @@ class Gaussian(Shape):
 
     def strDesc(self, delimiters=('"', "x", "@", "deg"), **kw):
         return """%.2g%s%s%.2g%s%s%d%s""" % (
-        self.ex * DEG * 3600, delimiters[0], delimiters[1], self.ey * DEG * 3600, delimiters[0],
-        delimiters[2], round(self.pa * DEG), delimiters[3])
+            self.ex * DEG * 3600, delimiters[0], delimiters[1], self.ey * DEG * 3600, delimiters[0],
+            delimiters[2], round(self.pa * DEG), delimiters[3])
 
     def strDescErr(self, delimiters=('"', "x", "@", "deg"), **kw):
         err = self.getShapeErr()
         return err and """%.2g%s%s%.2g%s%s%d%s""" % (
-        err[0] * DEG * 3600, delimiters[0], delimiters[1], err[1] * DEG * 3600, delimiters[0],
-        delimiters[2], round(err[2] * DEG), delimiters[3])
+            err[0] * DEG * 3600, delimiters[0], delimiters[1], err[1] * DEG * 3600, delimiters[0],
+            delimiters[2], round(err[2] * DEG), delimiters[3])
 
 
 class FITSImage(Shape):
@@ -457,8 +498,8 @@ startup_dprint(1, "end of class defs")
 # populate dict of AllowedTypes with all classes defined so far
 globs = list(globals().items())
 
-AllowedTypes = dict(iter(AtomicTypes.items()))
-AllowedTypes['NoneType'] = type(None);  # this must be a type, otherwise isinstance() doesn't work
+AllowedTypes = dict(iter(list(AtomicTypes.items())))
+AllowedTypes['NoneType'] = type(None)  # this must be a type, otherwise isinstance() doesn't work
 for name, val in globs:
     if isinstance(val, type):
         AllowedTypes[name] = val
